package jp.gr.java_conf.matsubara.expresser.scope;

import jp.gr.java_conf.matsubara.expresser.Expresser;
import jp.gr.java_conf.matsubara.expresser.IVariable;
import jp.gr.java_conf.matsubara.expresser.Variable;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.type.IExpObject;

/**
 * XR[v̎̈łB<br/>
 * ϐɑ΂ăV[PVANZXs܂B<br/>
 * offsetlȂł̃ANZX̏ꍇAScopeIuWFNgȂ܂A
 * fixedvpeBȂ߁ACӂ̎_offset擾邱Ƃł܂B<br/>
 * <br/> 
 * 쐬: 2005/06/05
 * 
 * @author m.matsubara
 */

public class SequentialScope implements IScope {
    private int[]  idArray;
    private IVariable[] varArray;
    private int count = 0;

    private IScope scopeChain = null;
    
    public SequentialScope() {
        this(10);
    }
    
    public SequentialScope(int initialSize) {
        idArray = new int[initialSize];
        varArray = new IVariable[initialSize];
    }
    
    
	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getVariable(java.lang.Integer)
	 */
	public IVariable getVariable(Integer id) throws ExpException {
        int idValue = id.intValue();
        for (int idx = 0; idx < count; idx++) {
            if (idArray[idx] == idValue)
                return varArray[idx];
        }
        if (scopeChain != null)
            return scopeChain.getVariable(id);
        return null;
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getVariable(java.lang.Integer, int)
	 */
	public IVariable getVariable(Integer id, int offset) throws ExpException {
        if (offset <= count && idArray[offset - 1] == id.intValue())
            return varArray[offset - 1];
        else if (scopeChain != null)
            return scopeChain.getVariable(id, offset);
        else
            return null;
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getValue(java.lang.Integer)
	 */
	public IExpObject getValue(Integer id) throws ExpException {
        int offset = getOffset(id);
        if (offset <= 0) {
            if (scopeChain != null)
                return getValue(id);
            else
                return Expresser.UNDEFINED;
        }
		IVariable variable = getVariable(id, offset);
        if (variable == null)
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
		return variable.getValue();
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getValue(java.lang.Integer, int)
	 */
	public IExpObject getValue(Integer id, int offset) throws ExpException {
        IVariable variable = getVariable(id, offset);
        if (variable == null)
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
        return variable.getValue();
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#setValue(java.lang.Integer, java.lang.Object)
	 */
	public void setValue(Integer id, IExpObject value) throws ExpException {
        IVariable variable = getVariable(id);
        if (variable == null)
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
        variable.setValue(value);
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#setValue(java.lang.Integer, int, java.lang.Object)
	 */
	public void setValue(Integer id, int offset, IExpObject value)
			throws ExpException {
        IVariable variable = getVariable(id, offset);
        if (variable == null)
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
        variable.setValue(value);
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#defineVariable(java.lang.Integer)
	 */
	public IVariable defineVariable(Integer id) throws ExpException {
        //  z񂪃I[o[t[Ƃ̑Ώ
        if (count >= idArray.length) {
            int[] idArrayWork = new int[count * 2];
            IVariable[] varArrayWork = new IVariable[count * 2];
            System.arraycopy(idArray, 0, idArrayWork, 0, count);
            System.arraycopy(varArray, 0, varArrayWork, 0, count);
            idArray = idArrayWork;
            varArray = varArrayWork;
        }
        IVariable variable = new Variable(Expresser.UNDEFINED);
        idArray[count] = id.intValue();
        varArray[count] = variable;
        count++;
        return variable;
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#setScopeChain(jp.gr.java_conf.matsubara.expresser.IScope)
	 */
	public void setScopeChain(IScope nextScope) {
		this.scopeChain = nextScope;
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getScopeChain()
	 */
	public IScope getScopeChain() {
		return this.scopeChain;
	}

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getOffset(java.lang.Integer)
	 */
	public int getOffset(Integer id) {
        int idValue = id.intValue();
		for (int idx = 0; idx < count; idx++) {
            if (idArray[idx] == idValue)
                return idx + 1;
        }
        if (scopeChain != null)
            return scopeChain.getOffset(id);
        return 0;
	}
}
