package jp.gr.java_conf.matsubara.interpreter.ecmascript.parser;

import jp.gr.java_conf.matsubara.expresser.Expresser;
import jp.gr.java_conf.matsubara.expresser.Range;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression;
import jp.gr.java_conf.matsubara.interpreter.ScriptEngine;
import jp.gr.java_conf.matsubara.interpreter.statement.BlockStatement;
import jp.gr.java_conf.matsubara.interpreter.statement.WhileStatement;

/**
 * do-while Xe[ggłB<br/>
 * 
 * 쐬: 2005/03/27
 * 
 * @author m.matsubara
 *
 */
public class DoWhileStatementParser implements IStatementParser {
    public String targetStatement() {
        return "do";
    }
    
    /* (Javadoc Ȃ)
     * @see jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.IStatementParser#parseStatement(jp.gr.java_conf.matsubara.interpreter.ScriptEngine, jp.gr.java_conf.matsubara.interpreter.statement.BlockStatement, java.lang.String, char[], int, int, java.lang.Integer)
     */
    public int parseStatement(ScriptEngine se, BlockStatement blockStatement, String statementWord, char[] script, int begin, int end, Integer labelID) throws ExpException {
        int idx = begin;
        int nStatementLength = statementWord.length();
        idx = Expresser.skipWhitespace(script, idx + nStatementLength, end);

        //  [v{
        BlockStatement loopStatement = new BlockStatement();
        idx = se.parseStatement1(loopStatement, script, idx, end);
        
        //  while L[[h
        idx = Expresser.skipWhitespace(script, idx, end);
        String s = se.getExpresser().nextToken(script, idx, end);
        if ("while".equals(s) == false) {
            throw new ExpException(ScriptEngine.getResString("NecessaryIdentifier", "while"), idx);
        }
        idx += s.length();

        //  
        int conditionBeginIdx = Expresser.skipWhitespace(script, idx, end);
        if (script[conditionBeginIdx] != '(')
            throw new ExpException(ScriptEngine.getResString("NecessaryParentheses"), conditionBeginIdx);
        int conditionEndIdx = Expresser.skipParenthesis(script, conditionBeginIdx, end);
        Range conditionRange = new Range(conditionBeginIdx + 1, conditionEndIdx - 1);
        IParsedExpression condition = se.getExpresser().parse(script, conditionBeginIdx + 1, conditionEndIdx - 1);

        idx = conditionEndIdx;
        
        
        WhileStatement statement = new WhileStatement(condition, conditionRange, loopStatement.getStatement(0), labelID);
        statement.setDoWhile(true);
        blockStatement.add(statement);
        
        return idx;
    }
}
