package jp.gr.java_conf.matsubara.interpreter.ecmascript.parser;

import jp.gr.java_conf.matsubara.expresser.Expresser;
import jp.gr.java_conf.matsubara.expresser.Range;
import jp.gr.java_conf.matsubara.expresser.StringToId;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression;
import jp.gr.java_conf.matsubara.expresser.parsedexpression.WrappedExpression;
import jp.gr.java_conf.matsubara.interpreter.ScriptEngine;
import jp.gr.java_conf.matsubara.interpreter.statement.BlockStatement;
import jp.gr.java_conf.matsubara.interpreter.statement.ForInStatement;
import jp.gr.java_conf.matsubara.interpreter.statement.ForStatement;
import jp.gr.java_conf.matsubara.interpreter.statement.VariableDefineStatement;

/**
 * for Xe[ggłB<br/>
 * 
 * 쐬: 2005/01/31
 * 
 * @author m.matsubara
 *
 */
public class ForStatementParser implements IStatementParser {
    public String targetStatement() {
        return "for";
    }
    
    /* (Javadoc Ȃ)
     * @see jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.IStatementParser#parseStatement(jp.gr.java_conf.matsubara.interpreter.ScriptEngine, jp.gr.java_conf.matsubara.interpreter.statement.BlockStatement, java.lang.String, char[], int, int, java.lang.Integer)
     */
    public int parseStatement(ScriptEngine se, BlockStatement blockStatement, String statementWord, char[] script, int begin, int end, Integer labelID) throws ExpException {
        int idx = begin;
        int statementLength = statementWord.length();

        int controlBeginIdx = Expresser.skipWhitespace(script, idx + statementLength, end);
        if (script[controlBeginIdx] != '(')
            throw new ExpException(ScriptEngine.getResString("NecessaryParentheses"), controlBeginIdx);
        int controlEndIdx = Expresser.skipParenthesis(script, controlBeginIdx, end);
        
        Range[] control = Expresser.split(';', script, controlBeginIdx + 1, controlEndIdx - 1);
        if (control.length != 3) {
            return parseForInStatement(se, blockStatement, statementWord, script, begin, end, labelID);
            //throw new ExpException(ScriptEngine.getResString("NecessarySemicolon"), controlBeginIdx + 1);
        }

        //  for s̉
        idx = controlEndIdx;
        BlockStatement loopStatement = new BlockStatement();
        idx = Expresser.skipWhitespace(script, idx, end);
        idx = se.parseStatement1(loopStatement, script, idx, end);

        //  [v̏
        IParsedExpression initialExpression = null;
        if (control[0].getBegin() < control[0].getEnd()) {
            String word = se.getExpresser().nextToken(script, control[0].getBegin(), control[0].getEnd());    
            if ("var".equals(word)) {
                VarStatementParser varParser = new VarStatementParser();
                varParser.parseStatement(se, blockStatement, statementWord, script, control[0].getBegin(), control[1].getBegin(), null);    //  control[0].getEnd() łȂ̂͂̓Z~RF邽
            } else
                initialExpression = se.getExpresser().parse(script, control[0]);
        }
        //  [v̌p
        IParsedExpression condition = null;
        if (control[1].getBegin() < control[1].getEnd())
            condition = se.getExpresser().parse(script, control[1]); 
        //  [vP񂲂ƂɎs鎮
        IParsedExpression loopEndExpression = null;
        if (control[2].getBegin() < control[2].getEnd())
            loopEndExpression = se.getExpresser().parse(script, control[2]); 
        ForStatement statement = new ForStatement(initialExpression, condition, loopEndExpression, loopStatement.getStatement(0), control[1].getBegin(), labelID);
        blockStatement.add(statement);
        
        return idx;
    }
    
    /**
     * For-in Xe[gg͂
     * @param se XNvgGW
     * @param block ̃ubNXe[gg̍Ōɉ͂ꂽXe[ggǉ
     * @param statementWird \͂Xe[gg̍ŏ̒P
     * @param script Xe[gg
     * @param begin \͊Jnʒu
     * @param end \͏Iʒu
     * @param labelID Xe[ggɃxtꍇIDEȂꍇ null
     * @return \͂̎̊Jnʒu
     * @throws ExpException \G[
     */
    public int parseForInStatement(ScriptEngine se, BlockStatement blockStatement, String statementWord, char[] script, int begin, int end, Integer labelID) throws ExpException {
        Expresser expresser = se.getExpresser();
        int idx = begin;
        int statementLength = statementWord.length();

        int controlBeginIdx = Expresser.skipWhitespace(script, idx + statementLength, end);
        if (script[controlBeginIdx] != '(')
            throw new ExpException(ScriptEngine.getResString("NecessaryParentheses"), controlBeginIdx);
        int controlEndIdx = Expresser.skipParenthesis(script, controlBeginIdx, end);
        idx = controlBeginIdx + 1;
        
        //  For-in  for (variable in object) ... `Ƃ
        
        boolean useVar = false;
        idx = Expresser.skipWhitespace(script, idx, controlEndIdx - 1);
        String variable = expresser.nextToken(script, idx, controlEndIdx - 1);
        
        if ((expresser.isIdentifier(variable) == false) && (variable.equals("var") == false)) 
            throw new ExpException(ScriptEngine.getResString("IllegalIdentifier", variable), idx);
        idx += variable.length();
        idx = Expresser.skipWhitespace(script, idx, controlEndIdx - 1);
        if (variable.equals("var")) {
        	useVar = true;
            variable = expresser.nextToken(script, idx, controlEndIdx - 1);
            idx += variable.length();
            idx = Expresser.skipWhitespace(script, idx, controlEndIdx - 1);
        }
        
        String inWord = expresser.nextToken(script, idx, controlEndIdx - 1);
        if (inWord.equals("in") == false)
            throw new ExpException(ScriptEngine.getResString("SyntaxError", variable), idx);
        idx += inWord.length();
        idx = Expresser.skipWhitespace(script, idx, controlEndIdx);
        
        IParsedExpression objectExpression = se.getExpresser().parse(script, idx, controlEndIdx - 1);
        Integer variableID = StringToId.toId(variable);
        
        //  for s̉
        idx = controlEndIdx;
        BlockStatement loopStatement = new BlockStatement();
        idx = Expresser.skipWhitespace(script, idx, end);
        idx = se.parseStatement1(loopStatement, script, idx, end);

        //  [v̏
        if (useVar) {
        	VariableDefineStatement varStatement = new VariableDefineStatement(variableID, new WrappedExpression(Expresser.UNDEFINED));
            blockStatement.add(varStatement);
        }
        ForInStatement statement = new ForInStatement(variableID, objectExpression, loopStatement, labelID);
        blockStatement.add(statement);
        
        return idx;
        
        
    }
}
